/* Copyright (c) 2006 Pearl Crescent, LLC.  All Rights Reserved. */

var gBrowserWindow = null; // TODO: ideally this would not be global
const kMinimumTimeout = 4;
var gLastTimeout = kMinimumTimeout;


function openBrowserWindow(aURI, aWindowFeatures)
{
  var argStr;
  if (aURI)
  {
    argStr = Components.classes["@mozilla.org/supports-string;1"]
             .getService(Components.interfaces.nsISupportsString);
    argStr.data = aURI.spec;
  }

  var prefSvc = Components.classes["@mozilla.org/preferences-service;1"]
                .getService(Components.interfaces.nsIPrefBranch);
  var browserChromeURL = prefSvc.getCharPref("browser.chromeURL");

  var wwSvc = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
              .getService(Components.interfaces.nsIWindowWatcher);
  var win = wwSvc.openWindow(null, browserChromeURL, "_blank",
                             "chrome,dialog=no,all" + aWindowFeatures, argStr);
  gBrowserWindow = win;

  if (win)
  {
    gLastTimeout = kMinimumTimeout;
    win.setTimeout(finishInitialization, gLastTimeout);
  }
}


function finishInitialization()
{
  try {
    var browser = this.top.getBrowser();
    if (browser.webProgress.isLoadingDocument || browser.contentDocument.location == "about:blank")
    {
      // If still loading, image will be saved from progress listener.
      browser.addProgressListener(gBrowserProgressListener,
                         Components.interfaces.nsIWebProgress.NOTIFY_LOCATION);
    }
    else
    {
      var win = gBrowserWindow;
      win.setTimeout("SavePageImage(true, true)", 0);
    }
  }
  catch (e)
  {
    if (gLastTimeout < 1000)
    {
      gLastTimeout *= 2; // double the timeout and try again soon.
      gBrowserWindow.setTimeout(finishInitialization, gLastTimeout);
    }
  }
}


// Returns null if aFlagName is not present.
// Returns "" if no argument was provided.  Never throws.
function getCmdLineArg(aCmdLine, aFlagName)
{
  var arg = "";
  try
  {
    arg = aCmdLine.handleFlagWithParam(aFlagName, false);
  }
  catch (e) {}

  return arg;
}


var gBrowserProgressListener = {
  kWebProgListener: Components.interfaces.nsIWebProgressListener,

  /* nsISupports implementation. */
  QueryInterface: function (aIID)
  {
    if (!aIID.equals(Components.interfaces.nsISupports) &&
        !aIID.equals(this.kWebProgListener))
      throw Components.errors.NS_ERROR_NO_INTERFACE;

    return this;
  },
  
  /* nsIWebProgressListener implementation */
  onStateChange: function (aWebProgress, aRequest, aStateFlags, aStatus)
  {
    if (gBrowserWindow && (aStateFlags & this.kWebProgListener.STATE_STOP) &&
        (aStateFlags & this.kWebProgListener.STATE_IS_WINDOW) &&
        (aRequest.name != "about:blank"))
    {
      try {
        gBrowserWindow.setTimeout("SavePageImage(true, true)", 0);
        gBrowserWindow = null; // Done.

        /*
         * Calling removeProgressListener() causes the browser window to
         * never finish loading the page.  SavePageImage() closes the window
         * anyway.
         */
      } catch (e) {}
    }
  },

  onProgressChange: function (aWebProgress, aRequest, aCurSelfProgress,
                             aMaxSelfProgress, aCurTotalProgress,
                             aMaxTotalProgress) {},

  onLocationChange: function (aWebProgress, aRequest, aLocURI) {},
  onStatusChange: function (aWebProgress, aRequest, aStatus, aMsg) {},
  onSecurityChange: function (aWebProgress, aRequest, aState) {},
  onLinkIconAvailable: function (a1, a2) {}
};


var gSavePNGCommandLineHandler = {
  /* nsISupports implementation. */
  QueryInterface: function (aIID)
  {
    if (!aIID.equals(Components.interfaces.nsISupports) &&
        !aIID.equals(Components.interfaces.nsIFactory) &&
        !aIID.equals(Components.interfaces.nsICommandLineHandler))
      throw Components.errors.NS_ERROR_NO_INTERFACE;

    return this;
  },

  /* nsIFactory implementation. */
  createInstance: function (aOuter, aIID)
  {
    if (null != aOuter)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
      
    return this.QueryInterface(aIID);
  },

  lockFactory: function (aDoLock) {},

  /* nsICommandLineHandler implementation. */
  handle: function (aCmdLine)
  {
    var uriArg = getCmdLineArg(aCmdLine, "savepng");
    if (null != uriArg)
    {
      if (0 == uriArg.length)
      {
        // TODO: No URL given.  Open and save the default page?
        return;
      }

      var uri = aCmdLine.resolveURI(uriArg);
      var windFeatures = "";
      var width = getCmdLineArg(aCmdLine, "width");
      if (width && width.length > 0)
        windFeatures += ",width=" + width;
      var height = getCmdLineArg(aCmdLine, "height");
      if (height && height.length > 0)
        windFeatures += ",height=" + height;

      openBrowserWindow(uri, windFeatures);
      aCmdLine.preventDefault = true;
    }
  },

  helpInfo : "  -savepng url        Save image of page as a PNG.\n"
};


var gSavePNGCommandLineHandlerModule =
{
  kCLHCategory: "m-savepng",
  kICompReg: Components.interfaces.nsIComponentRegistrar,
  kClassID: Components.ID("{bdaf0e90-14ab-4350-904c-c0a7bc0ce5a7}"),
  kContractID: "@pearlcrescent.com/SavePNGCommandLineHandler;1",

  /* nsISupports implementation. */
  QueryInterface: function (aIID)
  {
    if (aIID.equals(Components.interfaces.nsIModule) ||
        aIID.equals(Components.interfaces.nsISupports))
      return this;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  /* nsIModule implementation. */
  getClassObject: function (aCompMgr, aClassID, aIID)
  {
    if (!aClassID.equals(this.kClassID))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    return gSavePNGCommandLineHandler.QueryInterface(aIID);
  },

  registerSelf: function (aCompMgr, aFileSpec, aLocation, aType)
  {
    aCompMgr = aCompMgr.QueryInterface(this.kICompReg);
    aCompMgr.registerFactoryLocation(this.kClassID, "savePNGCLH",
                                 this.kContractID, aFileSpec, aLocation, aType);

    var catMgr = Components.classes["@mozilla.org/categorymanager;1"]
                 .getService(Components.interfaces.nsICategoryManager);
    catMgr.addCategoryEntry("command-line-handler", this.kCLHCategory,
                            this.kContractID, true, true);
  },

  unregisterSelf: function (aCompMgr, aFileSpec, aLocation)
  {
    aCompMgr = aCompMgr.QueryInterface(this.kICompReg);
    aCompMgr.unregisterFactoryLocation(this.kClassID, aFileSpec);
    var catMgr = Components.classes["@mozilla.org/categorymanager;1"]
                 .getService(Components.interfaces.nsICategoryManager);
    catMgr.deleteCategoryEntry("command-line-handler", this.kCLHCategory, true);
  },

  canUnload: function (aCompMgr) { return true; }
};
  

function NSGetModule(aCompMgr, aFileSpec)
{
  return gSavePNGCommandLineHandlerModule;
}
